@@ -5,16 +5,16 @@ use warnings;
 
 use base 'ObjectDB::Related';
 
-our $VERSION = '3.05';
+our $VERSION = '3.06';
 
 use Scalar::Util ();
+use ObjectDB::Util qw(merge);
 
 sub create_related {
     my $self = shift;
     my ($row, $related) = @_;
 
-    my $meta = $self->{meta};
-
+    my $meta = $self->meta;
     my ($from, $to) = %{$meta->map};
 
     my @params = ($to => $row->column($from));
@@ -25,8 +25,7 @@ sub create_related {
             push @objects, $related->set_columns(@params)->save;
         }
         else {
-            push @objects,
-              $meta->class->new->set_columns(%$related, @params)->create;
+            push @objects, $meta->class->new(%$related, @params)->create;
         }
     }
 
@@ -34,72 +33,49 @@ sub create_related {
 }
 
 sub find_related {
-    my $self   = shift;
-    my ($row)  = shift;
-    my %params = @_;
+    my $self = shift;
+    my ($row) = shift;
 
     my $meta = $self->meta;
-
-    $params{where} ||= [];
-
     my ($from, $to) = %{$meta->map};
 
     return unless defined $row->column($from);
 
-    push @{$params{where}}, ($to => $row->column($from));
-
-    return $meta->class->table->find(%params);
+    return $self->_related_table->find($self->_build_params($row, @_));
 }
 
 sub count_related {
     my $self = shift;
     my ($row) = shift;
 
-    my %params = @_;
-
-    my $meta = $self->meta;
-
-    my ($from, $to) = %{$meta->{map}};
-
-    push @{$params{where}}, ($to => $row->get_column($from));
-
-    if ($meta->{where}) {
-        push @{$params{where}}, %{$meta->{where}};
-    }
-
-    return $meta->class->table->count(%params);
+    return $self->_related_table->count($self->_build_params($row, @_));
 }
 
 sub update_related {
     my $self = shift;
     my ($row) = shift;
 
-    my %params = @_ == 1 ? %{$_[0]} : @_;
-
-    my $meta = $self->meta;
-
-    my ($from, $to) = %{$meta->map};
-    my $where = [$to => $row->get_column($from)];
-
-    push @$where, @{$params{where}} if $params{where};
-
-    return $meta->class->table->update(where => $where, @_);
+    return $self->_related_table->update($self->_build_params($row, @_));
 }
 
 sub delete_related {
     my $self = shift;
     my ($row) = shift;
 
-    my %params = @_;
-    $params{where} ||= [];
+    return $self->_related_table->delete($self->_build_params($row, @_));
+}
+
+sub _related_table { shift->meta->class->table }
+
+sub _build_params {
+    my $self = shift;
+    my ($row) = shift;
 
     my $meta = $self->meta;
-
     my ($from, $to) = %{$meta->map};
 
-    push @{$params{where}}, ($to => $row->get_column($from));
-
-    return $meta->class->table->delete(%params);
+    my $params = merge { @_ }, {where => [$to => $row->column($from)]};
+    return %$params;
 }
 
 1;
