@@ -5,9 +5,10 @@ use warnings;
 
 use base 'ObjectDB::Related::ManyToOne';
 
-our $VERSION = '3.05';
+our $VERSION = '3.06';
 
 use Scalar::Util ();
+use ObjectDB::Util qw(merge);
 
 sub create_related {
     my $self = shift;
@@ -17,8 +18,7 @@ sub create_related {
         Carp::croak('cannot create multiple related objects in one to one');
     }
 
-    my $meta = $self->{meta};
-
+    my $meta = $self->meta;
     my ($from, $to) = %{$meta->map};
 
     my @where = ($to => $row->column($from));
@@ -36,28 +36,30 @@ sub create_related {
 }
 
 sub update_related {
-    my $self   = shift;
-    my ($row)  = shift;
-    my %params = @_ == 1 ? %{$_[0]} : @_;
+    my $self = shift;
+    my ($row) = shift;
 
-    my $meta = $self->meta;
-
-    my ($from, $to) = %{$meta->map};
-    my $where = [$to => $row->get_column($from)];
-
-    return $meta->class->table->update(set => $params{set});
+    return $self->_related_table->update($self->_build_params($row, @_));
 }
 
 sub delete_related {
     my $self = shift;
     my ($row) = shift;
 
+    return $self->_related_table->delete($self->_build_params($row, @_));
+}
+
+sub _related_table { shift->meta->class->table }
+
+sub _build_params {
+    my $self = shift;
+    my ($row) = shift;
+
     my $meta = $self->meta;
-
     my ($from, $to) = %{$meta->map};
-    my $where = [$to => $row->get_column($from)];
 
-    return $meta->class->table->delete();
+    my $params = merge { @_ }, {where => [$to => $row->column($from)]};
+    return %$params;
 }
 
 1;
